import java.util.*;
import java.io.*;
import java.net.*;


public class IniFileIO {
public  int numberoflines=0;
private final int MAXLINES=65535;
private RandomAccessFile raf=null; 
private String mode;
private String filename;

private String[] Var = new String[MAXLINES];

	public void setVarBuffer(String[] vb) {
		Var = vb;
	}
	public void setNumberOfLines(int nol) {
		numberoflines=nol;
	}
	public String[] getVarBuffer() {
		return Var;
	}

	public IniFileIO(String fname, String md) {
		filename = fname;
		mode = md;
	}

	public void open() throws Exception {
		
		if(filename.length() < 1) {
			throw new Exception("Not a valid filename.");
		}
		
		String linein;

		try {

			try {
				raf = new RandomAccessFile(filename,mode);
			} catch(IOException e) {
				throw new Exception(e.toString());
			  }

			while((linein = raf.readLine())!=null) {
				Var[numberoflines]=linein;
				numberoflines++;
			} 

		} catch(Exception ioErr) {
			System.out.println(ioErr.toString());	
		  }
	}

	public void close() {
		try {
			raf.close();
		} catch(IOException e) {System.out.println(e.toString()); }
	}

	public int getVarLine(String varname) {
		for(int cnt=0; cnt<numberoflines; cnt++) {
			if((Var[cnt].toUpperCase()).indexOf(varname.toUpperCase().trim()) != -1) {
				return cnt;
			}
		}
		return -1;
	}

	public int getVarLine(String varname, int occurence) {
		for(int cnt=0; cnt<numberoflines; cnt++) {
			if((Var[cnt].toUpperCase()).indexOf(varname.toUpperCase().trim()) == 0) {
				occurence--;
				if(occurence == 0) {
					return cnt;
				}
			}
		}
		return -1;
	}

	public String getLine(String varname) {
		for(int cnt=0; cnt<numberoflines; cnt++) {
			if((Var[cnt].toUpperCase()).indexOf(varname.toUpperCase().trim()) != -1) {
				return Var[cnt];				
			}
		}
		return null;
	}

	public String getVar(String varname) {
		for(int cnt=0; cnt<numberoflines; cnt++) {
			if((Var[cnt].toUpperCase()).indexOf(varname.toUpperCase().trim()) != -1) {
				String TheLine = Var[cnt];				

				StringTokenizer st = new StringTokenizer(TheLine);
				if(st.countTokens() > 1) {
					st.nextToken();
					return st.nextToken(); //return the second string in line
				}
				else { return null; }	
			}
		}
		return null;
	}
	public String getType(String varname) {
		for(int cnt=0; cnt<numberoflines; cnt++) {
			if((Var[cnt].toUpperCase()).indexOf(varname.toUpperCase().trim()) != -1) {
				String TheLine = Var[cnt];				

				StringTokenizer st = new StringTokenizer(TheLine, "|");
				if(st.countTokens() > 1) {
					st.nextToken();
					return st.nextToken(); //return the second string in line
				}
				else { return null; }	
			}
		}
		return null;
	}
	public String getRange(String varname) {
		for(int cnt=0; cnt<numberoflines; cnt++) {
			if((Var[cnt].toUpperCase()).indexOf(varname.toUpperCase().trim()) != -1) {
				String TheLine = Var[cnt];				

				StringTokenizer st = new StringTokenizer(TheLine, "|");
				if(st.countTokens() > 2) {
					st.nextToken();
					st.nextToken();
					return st.nextToken(); //return the second string in line
				}
				else { return null; }	
			}
		}
		return null;
	}

	public String getLine(int index) {
		if(index <= numberoflines) {
			return Var[index];
		}
		else { return null; }
	}

	public String getLineSub(int index, int sub) {
		if(index > numberoflines) { return null; }
		String TheLine = Var[index];				

		StringTokenizer st = new StringTokenizer(TheLine);
		if(st.countTokens() >= sub) {
			String subst=null;
			for(int cnt=0; cnt<sub; cnt++) {
				subst  = new String(st.nextToken());
			}
			return subst;
		}
		return null;
	}

	public String getVarSub(String varname, int sub) {
		String TheLine = null; 
		for(int cnt=0; cnt<numberoflines; cnt++) {
			if((Var[cnt].toUpperCase()).indexOf(varname.toUpperCase().trim()) != -1) {
				TheLine = Var[cnt];				
			}
		}
		if(TheLine!=null) {

			StringTokenizer st = new StringTokenizer(TheLine);
			if(st.countTokens() >= sub) {
				String subst=null;
				for(int cnt=0; cnt<sub; cnt++) {
					subst  = new String(st.nextToken());
				}
				return subst;
			}
		}
		return null;
	}

	public String getComment(String varname) {
		for(int cnt=0; cnt<numberoflines; cnt++) {
			if((Var[cnt].toUpperCase()).indexOf(varname.toUpperCase().trim()) != -1) {
				String TheLine = Var[cnt];				

				StringTokenizer comst = new StringTokenizer(TheLine, ";");
				if(comst.countTokens() > 1) {
					comst.nextToken();
					String comment  = new String(comst.nextToken());
					return comment;
				}
				else { return null; }
			} 
		}
		return null;
	}
	public int getNumberOfVars(){
		return numberoflines;
	}

	public void updateVar(String varname, String value) throws Exception {
		for(int cnt=0; cnt<numberoflines; cnt++) {
			if((Var[cnt].toUpperCase()).indexOf(varname.toUpperCase().trim()) != -1) {
				String TheLine = Var[cnt];				

				StringTokenizer comst = new StringTokenizer(TheLine, ";");
				if(comst.countTokens() > 1) {
					comst.nextToken();
					String comment  = new String(comst.nextToken());
					Var[cnt] = new String(varname+"         "+value+"    "+";"+comment);
				}
				else {
					Var[cnt] = new String(varname+"         "+value);
				}
				return;
			}
		}
		throw new Exception("variable: "+varname+" not found."); 
	}

	public void updateLine(String varname, String line) throws Exception {
		for(int cnt=0; cnt<numberoflines; cnt++) {
			if((Var[cnt].toUpperCase()).indexOf(varname.toUpperCase().trim()) != -1) {
				Var[cnt]=line;				
				return;
			}
		}
		throw new Exception("variable: "+varname+" not found."); 
	}

	public void deleteLine(String varname) throws Exception {
		for(int cnt=0; cnt<numberoflines; cnt++) {
			if((Var[cnt].toUpperCase()).indexOf(varname.toUpperCase().trim()) != -1) {
				Var[cnt]=null;				
				return;
			}
		}
		throw new Exception("variable: "+varname+" not found."); 
	}

	public void updateComment(String varname, String comment) throws Exception {
		for(int cnt=0; cnt<numberoflines; cnt++) {
			if((Var[cnt].toUpperCase()).indexOf(varname.toUpperCase().trim()) != -1) {
				String TheLine = Var[cnt];				

				StringTokenizer st = new StringTokenizer(TheLine);
				if(st.countTokens() > 1) {
					st.nextToken();
					String value  = new String(st.nextToken());
					Var[cnt] = new String(varname+"                     "+value+"         "+";"+comment);
				}
				return;
			}
		}
		throw new Exception("variable: "+varname+" not found."); 
	}

	public void flushChanges() throws Exception {
			try {
				raf.seek(0);
			} catch(IOException e) { throw new Exception("error seeking to start of file."); }
			for(int cnt=0; cnt<numberoflines; cnt++) {
				try {
					if(Var[cnt]!=null) {
						String s = new String(Var[cnt]+"\n"); 
						raf.writeBytes(s);
					}
				} catch(IOException e) { throw new Exception("error flushing changes to file"); }
			}
	}

	public void appendLine(String line) throws Exception {
		if(numberoflines < MAXLINES) {
			numberoflines++;
			Var[numberoflines-1]= new String(line+"\r");
			return;
		}
		else { throw new Exception("max number of lines reached."); }
	}
}
